@@ -1155,6 +1155,9 @@ class Turba_Api extends Horde_Registry_Api
         // ...and ensure the default source is used as a default.
         if (!count($opts['sources'])) {
             $opts['sources'] = array(Turba::getDefaultAddressbook());
+            if (!empty($opts['fields']) && empty($opts['fields'][$opts['sources'][0]])) {
+                $opts['fields'][$opts['sources'][0]] = $opts['fields'];
+            }
         }
 
         $driver = $injector->getInstance('Turba_Factory_Driver');
@@ -1778,6 +1781,7 @@ class Turba_Api extends Horde_Registry_Api
                 $list = $sdriver->search($criterium, null, 'OR', array(), $strict ? array_keys($criterium) : array());
             } catch (Turba_Exception $e) {
                 Horde::log($e, 'ERR');
+                continue;
             }
 
             if ($list instanceof Turba_List) {
